#include "../../include/freenet/boardlistinserter.h"
#include "../../include/freenet/boardlistxml.h"

#ifdef XMEM
	#include <xmem.h>
#endif

BoardListInserter::BoardListInserter()
{
	Initialize();
}

BoardListInserter::BoardListInserter(FCPv2 *fcp):IIndexInserter<long>(fcp)
{
	Initialize();
}

void BoardListInserter::CheckForNeededInsert()
{
	// only do 1 insert at a time
	if(m_inserting.size()==0)
	{
		DateTime today;
		DateTime daysback;
		
		today.SetToGMTime();
		daysback.SetToGMTime();

		// 20 days
		daysback.Add(0,0,0,-20);

		// get identities who posted messages to boards in the past 20 days
		SQLite3DB::Statement st=m_db->Prepare("SELECT tblLocalIdentity.LocalIdentityID FROM tblLocalIdentity INNER JOIN tblMessageInserts ON tblLocalIdentity.LocalIdentityID=tblMessageInserts.LocalIdentityID WHERE tblLocalIdentity.PublishBoardList='true' AND (tblLocalIdentity.LastInsertedBoardList<? OR tblLocalIdentity.LastInsertedBoardList IS NULL) AND tblMessageInserts.Day>=? GROUP BY tblLocalIdentity.LocalIdentityID;");
		st.Bind(0,today.Format("%Y-%m-%d"));
		st.Bind(1,daysback.Format("%Y-%m-%d"));
		st.Step();

		if(st.RowReturned())
		{
			int localidentityid;
			st.ResultInt(0,localidentityid);
			StartInsert(localidentityid);
		}
	}
}

const bool BoardListInserter::HandlePutFailed(FCPMessage &message)
{
	std::vector<std::string> idparts;
	long localidentityid;
	long index;

	StringFunctions::Split(message["Identifier"],"|",idparts);
	StringFunctions::Convert(idparts[1],localidentityid);
	StringFunctions::Convert(idparts[2],index);

	if(message["Fatal"]=="true" || message["Code"]=="9")
	{
		SQLite3DB::Statement st=m_db->Prepare("INSERT INTO tblBoardListInserts(LocalIdentityID,Day,InsertIndex,Inserted) VALUES(?,?,?,'false');");
		st.Bind(0,localidentityid);
		st.Bind(1,idparts[4]);
		st.Bind(2,index);
		st.Step();
	}

	RemoveFromInsertList(localidentityid);

	return true;
}

const bool BoardListInserter::HandlePutSuccessful(FCPMessage &message)
{
	DateTime now;
	std::vector<std::string> idparts;
	long localidentityid;
	long index;

	StringFunctions::Split(message["Identifier"],"|",idparts);
	StringFunctions::Convert(idparts[1],localidentityid);
	StringFunctions::Convert(idparts[2],index);

	SQLite3DB::Statement st=m_db->Prepare("INSERT INTO tblBoardListInserts(LocalIdentityID,Day,InsertIndex,Inserted) VALUES(?,?,?,'true');");
	st.Bind(0,localidentityid);
	st.Bind(1,idparts[4]);
	st.Bind(2,index);
	st.Step();

	now.SetToGMTime();
	st=m_db->Prepare("UPDATE tblLocalIdentity SET LastInsertedBoardList=? WHERE LocalIdentityID=?;");
	st.Bind(0,now.Format("%Y-%m-%d %H:%M:%S"));
	st.Bind(1,localidentityid);
	st.Step();

	RemoveFromInsertList(localidentityid);

	m_log->WriteLog(LogFile::LOGLEVEL_DEBUG,"BoardListInserter::HandlePutSuccessful successfully inserted BoardList.");

	return true;
}

void BoardListInserter::Initialize()
{
	m_fcpuniquename="BoardListInserter";
}

void BoardListInserter::StartInsert(const long &localidentityid)
{
	DateTime daysback;
	DateTime now;
	BoardListXML xml;
	FCPMessage message;
	std::string data;
	std::string datasizestr;
	std::string privatekey="";
	int index;
	std::string indexstr="";
	std::string localidentityidstr;

	now.SetToGMTime();
	daysback.SetToGMTime();
	daysback.Add(0,0,0,-20);

	// get boards
	SQLite3DB::Statement st=m_db->Prepare("SELECT BoardName,BoardDescription FROM tblBoard INNER JOIN tblMessageBoard ON tblBoard.BoardID=tblMessageBoard.BoardID INNER JOIN tblMessage ON tblMessageBoard.MessageID=tblMessage.MessageID INNER JOIN tblMessageInserts ON tblMessage.MessageUUID=tblMessageInserts.MessageUUID WHERE tblMessageInserts.LocalIdentityID=? AND tblMessageInserts.Day>=? GROUP BY tblBoard.BoardID;");
	st.Bind(0,localidentityid);
	st.Bind(1,daysback.Format("%Y-%m-%d"));
	st.Step();

	while(st.RowReturned())
	{
		std::string name="";
		std::string description="";

		st.ResultText(0,name);
		st.ResultText(1,description);

		xml.AddBoard(name,description);

		st.Step();
	}

	// get public key
	st=m_db->Prepare("SELECT PrivateKey FROM tblLocalIdentity WHERE LocalIdentityID=?;");
	st.Bind(0,localidentityid);
	st.Step();
	if(st.RowReturned())
	{
		st.ResultText(0,privatekey);
	}

	// get last index
	index=0;
	st=m_db->Prepare("SELECT MAX(InsertIndex) FROM tblBoardListInserts WHERE LocalIdentityID=? AND Day=?;");
	st.Bind(0,localidentityid);
	st.Bind(1,now.Format("%Y-%m-%d"));
	st.Step();
	if(st.RowReturned())
	{
		if(st.ResultNull(0)==false)
		{
			st.ResultInt(0,index);
			index++;
		}
	}
	StringFunctions::Convert(index,indexstr);

	data=xml.GetXML();
	StringFunctions::Convert(data.size(),datasizestr);
	StringFunctions::Convert(localidentityid,localidentityidstr);

	message.SetName("ClientPut");
	message["URI"]=privatekey+m_messagebase+"|"+now.Format("%Y-%m-%d")+"|BoardList|"+indexstr+".xml";
	message["Identifier"]=m_fcpuniquename+"|"+localidentityidstr+"|"+indexstr+"|"+message["URI"];
	message["UploadFrom"]="direct";
	message["DataLength"]=datasizestr;
	m_fcp->SendMessage(message);
	m_fcp->SendRaw(data.c_str(),data.size());

	m_inserting.push_back(localidentityid);

}
